# -*- coding: utf-8 -*-

"""
This module implements the core OOP interface :class:`S3Path`.

Import::

    >>> from s3pathlib.core import S3Path
    # or
    >>> from s3pathlib import S3Path
"""

from .s3path import S3Path
